@@ -22,160 +22,142 @@ use vars qw($VERSION @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS $AUTOLOAD);
 # will save memory.
 
 my @constants_names = (
-        qw(SG_ERROR_NONE SG_ERROR_INVALID_ARGUMENT SG_ERROR_ASPRINTF
-          SG_ERROR_SPRINTF SG_ERROR_DEVICES SG_ERROR_DEVSTAT_GETDEVS
-          SG_ERROR_DEVSTAT_SELECTDEVS SG_ERROR_DISKINFO SG_ERROR_ENOENT
-          SG_ERROR_GETIFADDRS SG_ERROR_GETMNTINFO SG_ERROR_GETPAGESIZE
-          SG_ERROR_HOST SG_ERROR_KSTAT_DATA_LOOKUP SG_ERROR_KSTAT_LOOKUP
-          SG_ERROR_KSTAT_OPEN SG_ERROR_KSTAT_READ SG_ERROR_KVM_GETSWAPINFO
-          SG_ERROR_KVM_OPENFILES SG_ERROR_MALLOC SG_ERROR_MEMSTATUS
-          SG_ERROR_OPEN SG_ERROR_OPENDIR SG_ERROR_READDIR SG_ERROR_PARSE
-          SG_ERROR_PDHADD SG_ERROR_PDHCOLLECT SG_ERROR_PDHOPEN SG_ERROR_PDHREAD
-          SG_ERROR_PERMISSION SG_ERROR_PSTAT SG_ERROR_SETEGID SG_ERROR_SETEUID
-          SG_ERROR_SETMNTENT SG_ERROR_SOCKET SG_ERROR_SWAPCTL SG_ERROR_SYSCONF
-          SG_ERROR_SYSCTL SG_ERROR_SYSCTLBYNAME SG_ERROR_SYSCTLNAMETOMIB
-          SG_ERROR_SYSINFO SG_ERROR_MACHCALL SG_ERROR_IOKIT SG_ERROR_UNAME
-          SG_ERROR_UNSUPPORTED SG_ERROR_XSW_VER_MISMATCH SG_ERROR_GETMSG
-          SG_ERROR_PUTMSG SG_ERROR_INITIALISATION SG_ERROR_MUTEX_LOCK
-          SG_ERROR_MUTEX_UNLOCK),
-        qw(sg_unknown_configuration sg_physical_host sg_virtual_machine
-          sg_paravirtual_machine sg_hardware_virtualized),
-        qw(sg_fs_unknown sg_fs_regular sg_fs_special sg_fs_loopback
-          sg_fs_remote sg_fs_local sg_fs_alltypes),
-        qw(SG_IFACE_DUPLEX_FULL SG_IFACE_DUPLEX_HALF SG_IFACE_DUPLEX_UNKNOWN),
-        qw(SG_IFACE_DOWN SG_IFACE_UP),
-        qw(SG_PROCESS_STATE_RUNNING SG_PROCESS_STATE_SLEEPING
-          SG_PROCESS_STATE_STOPPED SG_PROCESS_STATE_ZOMBIE
-          SG_PROCESS_STATE_UNKNOWN),
-      );
+    qw(SG_ERROR_NONE SG_ERROR_INVALID_ARGUMENT SG_ERROR_ASPRINTF
+      SG_ERROR_SPRINTF SG_ERROR_DEVICES SG_ERROR_DEVSTAT_GETDEVS
+      SG_ERROR_DEVSTAT_SELECTDEVS SG_ERROR_DISKINFO SG_ERROR_ENOENT
+      SG_ERROR_GETIFADDRS SG_ERROR_GETMNTINFO SG_ERROR_GETPAGESIZE
+      SG_ERROR_HOST SG_ERROR_KSTAT_DATA_LOOKUP SG_ERROR_KSTAT_LOOKUP
+      SG_ERROR_KSTAT_OPEN SG_ERROR_KSTAT_READ SG_ERROR_KVM_GETSWAPINFO
+      SG_ERROR_KVM_OPENFILES SG_ERROR_MALLOC SG_ERROR_MEMSTATUS
+      SG_ERROR_OPEN SG_ERROR_OPENDIR SG_ERROR_READDIR SG_ERROR_PARSE
+      SG_ERROR_PDHADD SG_ERROR_PDHCOLLECT SG_ERROR_PDHOPEN SG_ERROR_PDHREAD
+      SG_ERROR_PERMISSION SG_ERROR_PSTAT SG_ERROR_SETEGID SG_ERROR_SETEUID
+      SG_ERROR_SETMNTENT SG_ERROR_SOCKET SG_ERROR_SWAPCTL SG_ERROR_SYSCONF
+      SG_ERROR_SYSCTL SG_ERROR_SYSCTLBYNAME SG_ERROR_SYSCTLNAMETOMIB
+      SG_ERROR_SYSINFO SG_ERROR_MACHCALL SG_ERROR_IOKIT SG_ERROR_UNAME
+      SG_ERROR_UNSUPPORTED SG_ERROR_XSW_VER_MISMATCH SG_ERROR_GETMSG
+      SG_ERROR_PUTMSG SG_ERROR_INITIALISATION SG_ERROR_MUTEX_LOCK
+      SG_ERROR_MUTEX_UNLOCK),
+    qw(sg_unknown_configuration sg_physical_host sg_virtual_machine
+      sg_paravirtual_machine sg_hardware_virtualized),
+    qw(sg_fs_unknown sg_fs_regular sg_fs_special sg_fs_loopback
+      sg_fs_remote sg_fs_local sg_fs_alltypes),
+    qw(SG_IFACE_DUPLEX_FULL SG_IFACE_DUPLEX_HALF SG_IFACE_DUPLEX_UNKNOWN),
+    qw(SG_IFACE_DOWN SG_IFACE_UP),
+    qw(SG_PROCESS_STATE_RUNNING SG_PROCESS_STATE_SLEEPING
+      SG_PROCESS_STATE_STOPPED SG_PROCESS_STATE_ZOMBIE
+      SG_PROCESS_STATE_UNKNOWN),
+);
 
-%EXPORT_TAGS = ( 'all' => [ qw(
-	get_error drop_privileges 
-	get_host_info 
-	get_cpu_stats
-	get_disk_io_stats
-	get_fs_stats
-	get_load_stats
-	get_mem_stats
-	get_swap_stats
-	get_network_io_stats
-	get_network_iface_stats
-	get_page_stats
-	get_user_stats
-	get_process_stats),
-	@constants_names
-    ] );
+my @stats = qw(
+  get_host_info
+  get_cpu_stats
+  get_disk_io_stats
+  get_fs_stats
+  get_load_stats
+  get_mem_stats
+  get_swap_stats
+  get_network_io_stats
+  get_network_iface_stats
+  get_page_stats
+  get_user_stats
+  get_process_stats
+);
+
+%EXPORT_TAGS = (
+    all       => [ qw(get_error drop_privileges), @stats, @constants_names ],
+    stats     => [@stats],
+    constants => [@constants_names],
+);
 
 @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );
+@EXPORT    = ( @{ $EXPORT_TAGS{'all'} } );
 
-@EXPORT = (qw(
-	get_error drop_privileges 
-	get_host_info 
-	get_cpu_stats
-	get_disk_io_stats
-	get_fs_stats
-	get_load_stats
-	get_mem_stats
-	get_swap_stats
-	get_network_io_stats
-	get_network_iface_stats
-	get_page_stats
-	get_user_stats
-	get_process_stats),
-	@constants_names
-    );
+$VERSION = '0.108_001';
 
-$VERSION = '0.105_001';
-
-sub AUTOLOAD {
+sub AUTOLOAD
+{
     # This AUTOLOAD is used to 'autoload' constants from the constant()
     # XS function.
 
     my $constname;
-    ($constname = $AUTOLOAD) =~ s/.*:://;
-    croak "&Unix::Statgrab::constant not defined" if $constname eq 'constant';
-    my ($error, $val) = constant($constname);
-    if ($error) { croak $error; }
+    ( $constname = $AUTOLOAD ) =~ s/.*:://;
+    $constname eq 'constant' and croak "&Unix::Statgrab::constant not defined";
+    my ( $error, $val ) = constant($constname);
+    $error and croak $error;
     {
-	no strict 'refs';
-	*$AUTOLOAD = sub { $val };
+        no strict 'refs';
+        *$AUTOLOAD = sub { $val };
     }
     goto &$AUTOLOAD;
 }
 
 bootstrap Unix::Statgrab $VERSION;
 
-my $to_hash_array = sub {
-    my ($self, @keys) = @_;
-    my $entries = $self->entries;
-    my @ary_hash;
-    foreach my $entry (0 .. $entries - 1)
-    {
-	my %kv = map { $_ => $self->$_($entry); } @keys;
-	push(@ary_hash, \%kv);
-    }
-    return @ary_hash;
-};
+sub Unix::Statgrab::sg_cpu_stats::as_list { @{$_[0]->fetchall_hashref}; }
+sub Unix::Statgrab::sg_cpu_stats::fetchall_hash { @{$_[0]->fetchall_hashref}; }
+sub Unix::Statgrab::sg_cpu_stats::fetchall_array { @{$_[0]->fetchall_arrayref}; }
+sub Unix::Statgrab::sg_cpu_stats::fetchall_table { ( $_[0]->colnames, @{$_[0]->fetchall_arrayref} ) }
 
-my @sg_cpu_stats_attrs = (qw(user kernel idle iowait swap nice total),
-	qw(context_switches voluntary_context_switches involuntary_context_switches syscalls),
-	qw(interrupts soft_interrupts systime));
+sub Unix::Statgrab::sg_cpu_percents::as_list { @{$_[0]->fetchall_hashref}; }
+sub Unix::Statgrab::sg_cpu_percents::fetchall_hash { @{$_[0]->fetchall_hashref}; }
+sub Unix::Statgrab::sg_cpu_percents::fetchall_array { @{$_[0]->fetchall_arrayref}; }
+sub Unix::Statgrab::sg_cpu_percents::fetchall_table { ( $_[0]->colnames, @{$_[0]->fetchall_arrayref} ) }
 
-sub Unix::Statgrab::sg_cpu_stats::as_list { return $to_hash_array->( $_[0], @sg_cpu_stats_attrs ); }
+sub Unix::Statgrab::sg_host_info::as_list { @{$_[0]->fetchall_hashref}; }
+sub Unix::Statgrab::sg_host_info::fetchall_hash { @{$_[0]->fetchall_hashref}; }
+sub Unix::Statgrab::sg_host_info::fetchall_array { @{$_[0]->fetchall_arrayref}; }
+sub Unix::Statgrab::sg_host_info::fetchall_table { ( $_[0]->colnames, @{$_[0]->fetchall_arrayref} ) }
 
-my @sg_cpu_percent_attrs = (qw(user kernel idle iowait swap nice time_taken));
+sub Unix::Statgrab::sg_disk_io_stats::as_list { @{$_[0]->fetchall_hashref}; }
+sub Unix::Statgrab::sg_disk_io_stats::fetchall_hash { @{$_[0]->fetchall_hashref}; }
+sub Unix::Statgrab::sg_disk_io_stats::fetchall_array { @{$_[0]->fetchall_arrayref}; }
+sub Unix::Statgrab::sg_disk_io_stats::fetchall_table { ( $_[0]->colnames, @{$_[0]->fetchall_arrayref} ) }
 
-sub Unix::Statgrab::sg_cpu_percents::as_list { return $to_hash_array->( $_[0], @sg_cpu_percent_attrs ); }
+sub Unix::Statgrab::sg_fs_stats::as_list { @{$_[0]->fetchall_hashref}; }
+sub Unix::Statgrab::sg_fs_stats::fetchall_hash { @{$_[0]->fetchall_hashref}; }
+sub Unix::Statgrab::sg_fs_stats::fetchall_array { @{$_[0]->fetchall_arrayref}; }
+sub Unix::Statgrab::sg_fs_stats::fetchall_table { ( $_[0]->colnames, @{$_[0]->fetchall_arrayref} ) }
 
-my @sg_host_info_attrs = (qw(os_name os_release os_version platform hostname),
-qw(bitwidth host_state ncpus maxcpus uptime systime));
+sub Unix::Statgrab::sg_load_stats::as_list { @{$_[0]->fetchall_hashref}; }
+sub Unix::Statgrab::sg_load_stats::fetchall_hash { @{$_[0]->fetchall_hashref}; }
+sub Unix::Statgrab::sg_load_stats::fetchall_array { @{$_[0]->fetchall_arrayref}; }
+sub Unix::Statgrab::sg_load_stats::fetchall_table { ( $_[0]->colnames, @{$_[0]->fetchall_arrayref} ) }
 
-sub Unix::Statgrab::sg_host_info::as_list { return $to_hash_array->( $_[0], @sg_host_info_attrs ); }
+sub Unix::Statgrab::sg_mem_stats::as_list { @{$_[0]->fetchall_hashref}; }
+sub Unix::Statgrab::sg_mem_stats::fetchall_hash { @{$_[0]->fetchall_hashref}; }
+sub Unix::Statgrab::sg_mem_stats::fetchall_array { @{$_[0]->fetchall_arrayref}; }
+sub Unix::Statgrab::sg_mem_stats::fetchall_table { ( $_[0]->colnames, @{$_[0]->fetchall_arrayref} ) }
 
-my @sg_disk_io_stats_attrs = (qw(disk_name read_bytes write_bytes systime));
+sub Unix::Statgrab::sg_swap_stats::as_list { @{$_[0]->fetchall_hashref}; }
+sub Unix::Statgrab::sg_swap_stats::fetchall_hash { @{$_[0]->fetchall_hashref}; }
+sub Unix::Statgrab::sg_swap_stats::fetchall_array { @{$_[0]->fetchall_arrayref}; }
+sub Unix::Statgrab::sg_swap_stats::fetchall_table { ( $_[0]->colnames, @{$_[0]->fetchall_arrayref} ) }
 
-sub Unix::Statgrab::sg_disk_io_stats::as_list { return $to_hash_array->( $_[0], @sg_disk_io_stats_attrs ); }
+sub Unix::Statgrab::sg_network_io_stats::as_list { @{$_[0]->fetchall_hashref}; }
+sub Unix::Statgrab::sg_network_io_stats::fetchall_hash { @{$_[0]->fetchall_hashref}; }
+sub Unix::Statgrab::sg_network_io_stats::fetchall_array { @{$_[0]->fetchall_arrayref}; }
+sub Unix::Statgrab::sg_network_io_stats::fetchall_table { ( $_[0]->colnames, @{$_[0]->fetchall_arrayref} ) }
 
-my @sg_fs_stats_attrs = (qw(device_name fs_type mnt_point device_type),
-	qw(size used free avail total_inodes used_inodes free_inodes avail_inodes),
-	qw(io_size block_size total_blocks free_blocks used_blocks avail_blocks systime));
+sub Unix::Statgrab::sg_network_iface_stats::as_list { @{$_[0]->fetchall_hashref}; }
+sub Unix::Statgrab::sg_network_iface_stats::fetchall_hash { @{$_[0]->fetchall_hashref}; }
+sub Unix::Statgrab::sg_network_iface_stats::fetchall_array { @{$_[0]->fetchall_arrayref}; }
+sub Unix::Statgrab::sg_network_iface_stats::fetchall_table { ( $_[0]->colnames, @{$_[0]->fetchall_arrayref} ) }
 
-sub Unix::Statgrab::sg_fs_stats::as_list { return $to_hash_array->( $_[0], @sg_fs_stats_attrs ); }
+sub Unix::Statgrab::sg_page_stats::as_list { @{$_[0]->fetchall_hashref}; }
+sub Unix::Statgrab::sg_page_stats::fetchall_hash { @{$_[0]->fetchall_hashref}; }
+sub Unix::Statgrab::sg_page_stats::fetchall_array { @{$_[0]->fetchall_arrayref}; }
+sub Unix::Statgrab::sg_page_stats::fetchall_table { ( $_[0]->colnames, @{$_[0]->fetchall_arrayref} ) }
 
-my @sg_load_stats_attrs = (qw(min1 min5 min15 systime));
+sub Unix::Statgrab::sg_user_stats::as_list { @{$_[0]->fetchall_hashref}; }
+sub Unix::Statgrab::sg_user_stats::fetchall_hash { @{$_[0]->fetchall_hashref}; }
+sub Unix::Statgrab::sg_user_stats::fetchall_array { @{$_[0]->fetchall_arrayref}; }
+sub Unix::Statgrab::sg_user_stats::fetchall_table { ( $_[0]->colnames, @{$_[0]->fetchall_arrayref} ) }
 
-sub Unix::Statgrab::sg_load_stats::as_list { return $to_hash_array->( $_[0], @sg_load_stats_attrs ); }
-
-my @sg_mem_stats_attrs = (qw(total free used cache systime));
-
-sub Unix::Statgrab::sg_mem_stats::as_list { return $to_hash_array->( $_[0], @sg_mem_stats_attrs ); }
-
-my @sg_swap_stats_attrs = (qw(total free used systime));
-
-sub Unix::Statgrab::sg_swap_stats::as_list { return $to_hash_array->( $_[0], @sg_swap_stats_attrs ); }
-
-my @sg_network_io_stats_attrs = (qw(interface_name tx rx ipackets opackets),
-	qw(ierrors oerrors collisions systime));
-
-sub Unix::Statgrab::sg_network_io_stats::as_list { return $to_hash_array->( $_[0], @sg_network_io_stats_attrs ); }
-
-my @sg_network_iface_stats_attrs = (qw(interface_name speed factor duplex up systime));
-
-sub Unix::Statgrab::sg_network_iface_stats::as_list { return $to_hash_array->( $_[0], @sg_network_iface_stats_attrs ); }
-
-my @sg_page_stats_attrs = (qw(pages_pagein pages_pageout systime));
-
-sub Unix::Statgrab::sg_page_stats::as_list { return $to_hash_array->( $_[0], @sg_page_stats_attrs ); }
-
-my @sg_user_stats_attrs = (qw(login_name record_id device hostname pid login_time systime));
-
-sub Unix::Statgrab::sg_user_stats::as_list { return $to_hash_array->( $_[0], @sg_user_stats_attrs ); }
-
-my @sg_process_stats_attrs = (qw(process_name proctitle pid parent pgid sessid uid euid gid egid),
-	qw(context_switches voluntary_context_switches involuntary_context_switches),
-	qw(proc_size proc_resident start_time time_spent cpu_percent nice state systime));
-
-sub Unix::Statgrab::sg_process_stats::as_list { return $to_hash_array->( $_[0], @sg_process_stats_attrs ); }
+sub Unix::Statgrab::sg_process_stats::as_list { @{$_[0]->fetchall_hashref}; }
+sub Unix::Statgrab::sg_process_stats::fetchall_hash { @{$_[0]->fetchall_hashref}; }
+sub Unix::Statgrab::sg_process_stats::fetchall_array { @{$_[0]->fetchall_arrayref}; }
+sub Unix::Statgrab::sg_process_stats::fetchall_table { ( $_[0]->colnames, @{$_[0]->fetchall_arrayref} ) }
 
 # Preloaded methods go here.
 
@@ -184,7 +166,6 @@ sub Unix::Statgrab::sg_process_stats::as_list { return $to_hash_array->( $_[0],
 1;
 __END__
 
-
 =head1 NAME
 
 Unix::Statgrab - Perl extension for collecting information about the machine
@@ -254,12 +235,52 @@ containing each attribute / value pair of available attributes.
 
 Provides the number of statistic objects in the returned collection.
 
-=item * C<as_list>
+=item * C<colnames>
 
-Provides a list containing each statistic entry as a hash with the
+Provides an arrayref containing each statistic attribute name in the
+same order as it's position in the stat-struct.
+
+=item * C<fetchrow_arrayref(offset = 0)>
+
+Provides an arrayref containing each statistic entry of the specified
+row in the same order as it's position in the stat-struct.
+
+=item * C<fetchall_arrayref>
+
+Provides an arrayref containing an item for each row as an arrayref.
+
+=item * C<fetchall_array>
+
+Provides list containing an item for each row as an arrayref.
+
+=item * C<fetchall_table>
+
+Provides list containing an item for each row as an arrayref headed
+by the arrayref containing the column names.
+
+=item * C<fetchrow_hashref(offset = 0)>
+
+Provides a hashref containing each statistic entry as a hash with the
+attribute names as key and the statistic attribute as appropriate
+value of the specified row.
+
+=item * C<fetchall_hashref>
+
+Provides an arrayref containing each statistic entry as a hashref with
+the attribute names as key and the statistic attribute as appropriate
+value.
+
+=item * C<fetchall_hash>
+
+Provides a list containing each statistic entry as a hashref with the
 attribute names as key and the statistic attribute as appropriate
 value.
 
+=item * C<as_list>
+
+B<Deprecated> alias for C<fetchall_hash>. Don't use in new projects and
+move away from it in existing code. Will be removed after 2020-12-31.
+
 =item * C<systime>
 
 Even if it's always an attribute (beside for C<cpu_percents>), returns
@@ -317,7 +338,7 @@ Returns L<Unix::Statgrab::sg_cpu_percents|http://www.i-scream.org/libstatgrab/do
 =head2 get_disk_io_stats
 
 Delivers the disk IO per disk stored in the kernel which holds the amount of
-data transferred since bootup. The object it returns is a
+data transferred since boot. The object it returns is a
 L<Unix::Statgrab::sg_disk_io_stats|http://www.i-scream.org/libstatgrab/docs/sg_get_disk_io_stats.3.html>.
 
 =head3 extra sg_disk_io_stats methods
@@ -587,7 +608,7 @@ If you have a bug report, a patch or a suggestion, please open a new
 report ticket at CPAN (but please check previous reports first in case
 your issue has already been addressed). You can mail any of the module
 maintainers, but you are more assured of an answer by posting to
-the dbi-users list or reporting the issue in RT.
+the i-scream-users list or reporting the issue in RT.
 
 Report tickets should contain a detailed description of the bug or
 enhancement request and at least an easily verifiable way of
@@ -619,7 +640,7 @@ Jens Rehsack, E<lt>rehsack AT cpan.orgE<gt>
 
 Copyright (C) 2004-2005 by Tassilo von Parseval
 
-Copyright (C) 2012-2013 by Jens Rehsack
+Copyright (C) 2012-2015 by Jens Rehsack
 
 This library is free software; you can redistribute it and/or modify it under
 the terms of the GNU Lesser General Public License as published by the Free
